/*
** ===================================================================
** Nzev souboru   : servo_c.c
** Mikrokontroler  : ATmega16
** Popis           : Funkce pro ovldn modelskho serva
**					
**
** Autor           : Radek Balada ( xbalad01@stud.feec.vutbr.cz )
** Pedmt         : Mikropocitace pro pristrojove aplikace
** ===================================================================
*/


/*
** Hlavikov soubory
*/
#include <avr/io.h>			// hlavikov soubor pro mikrokontrolr ATmega16
#define F_CPU 16000000UL	// frekvence krystalu
#include <avr\interrupt.h> 	//definicni soubor pro obsluhu preruseni
#include "servo_h.h" 		//definicni soubor pro ovladani serva


/*
** Globln promnn
*/
char timer = 125;
char poloha0  = 0b01111101;
char poloha1  = 0b01111101;
char poloha2  = 0b01111101;
char poloha3  = 0b01111101;
char poloha4  = 0b01111101;
char poloha5  = 0b01111101;
char poloha6  = 0b01111101;
char poloha7  = 0b01111101;


/*
** ===================================================================
**     Nzev funkce  : servo_init
**
**     Popis         :
**         	nastaveni vystupniho portu; inicializace citace/casovace0;
**			povoleni preruseni od citace/casovace0 
**
**     Parametry     : temp vychozi poloha serv (neutral)
**     Vst. hodnota : dn
** ===================================================================
*/
void servo_init( char temp )
{
	DDR( SERVO_PORT ) = 0b11111111;//deklarace vstupnho portu pro serva

	// Nastaveni Citace Casovace 0
	TCCR0 = (1<<WGM01)|(1<<CS01)|(1<<CS00);	//nastaveni citace/casovace 0
	OCR0 = 250;
	TIMSK = (1<<OCIE0);			//povoleni preruseni od citace/casovace 0 
	
	//nastaveni vychozich poloh serv
	poloha0  = temp;
	poloha1  = temp;
 	poloha2  = temp;
	poloha3  = temp;
	poloha4  = temp;
	poloha5  = temp;
	poloha6  = temp;
	poloha7  = temp;  
}



/*
** ===================================================================
**     Nzev funkce  : servo_poloha
**
**     Popis         :
**				funkce priradi vybranemu kanalu pozadovanou polohu serva,
**				polohu lze menit pouze u jednoho vybraneho kanalu serva
**
**     Parametry     : temp2 pozadovana poloha serva (0 az 250)
**					   temp3 cislo kanalu (0 az 7)
**     Vst. hodnota : dn
** ===================================================================
*/
void servo_poloha( char temp2, char temp3 )
{
switch ( temp3 )
	{
	case 0:		//kanal 0
		{	
		poloha0  = temp2;
		break;
		}
	case 1:		//kanal 1
		{
		poloha1  = temp2;
		break;
		}
	case 2:		//kanal 2
		{
 		poloha2  = temp2;
		break;
		}
	case 3:		//kanal 3
		{
		poloha3  = temp2;
		break;
		}
	case 4:		//kanal 4
		{
		poloha4  = temp2;
		break;
		}
	case 5:		//kanal 5
		{
		poloha5  = temp2;
		break;
		}
	case 6:		//kanal 6
		{
		poloha6  = temp2;
		break;
		}
	case 7:		//kanal 7
		{
		poloha7  = temp2;
		break;
		}
	}  
}




/*
** ===================================================================
**     Nzev funkce  : ISR( TIMER0_COMP_vect )
**
**     Popis         :
**         	obsluha preruseni citace/casovace0
**			peiodicky nastavuje vystupni piny 0 az 7 portu pro ovladani
**			8 nezavislych kanalu serv
**     Parametry     : dn
**     Vst. hodnota : dn
** ===================================================================
*/
ISR( TIMER0_COMP_vect )
{
// casovani PWM
timer++;
switch (timer)
	{
	//servo 0
	case 0:					// t=0ms
		{
		output0hi;
		OCR0 = 250;
		break;
		}
	case 1:					// t=1ms
		{
		output0hi;
		OCR0 = poloha0;
		break;
		}
	case 2:
		{
		output0low;
		OCR0 = 253-poloha0;
		break;
		}
	//servo 1
	case 3:					// t=2ms
		{
		output1hi;
		OCR0 = 250;
		break;
		}
	case 4:					// t=3ms
		{
		output1hi;
		OCR0 = poloha1;
		break;
		}
	case 5:
		{
		output1low;
		OCR0 = 253-poloha1;
		break;
		}
	//servo 2
	case 6:					// t=4ms
		{
		output2hi;
		OCR0 = 250;
		break;
		}
	case 7:					// t=5ms
		{
		output2hi;
		OCR0 = poloha2;
		break;
		}
	case 8:
		{
		output2low;
		OCR0 = 253-poloha2;
		break;
		}
	//servo 3
	case 9:					// t=6ms
		{
		output3hi;
		OCR0 = 250;
		break;
		}
	case 10:				// t=7ms
		{
		output3hi;
		OCR0 = poloha3;
		break;
		}
	case 11:
		{
		output3low;
		OCR0 = 253-poloha3;
		break;
		}
	//servo 4
	case 12:				// t=8ms
		{
		output4hi;
		OCR0 = 250;
		break;
		}
	case 13:				// t=9ms
		{
		output4hi;
		OCR0 = poloha4;
		break;
		}
	case 14:
		{
		output4low;
		OCR0 = 253-poloha4;
		break;
		}
	//servo 5
	case 15:				// t=10ms
		{
		output5hi;
		OCR0 = 250;
		break;
		}
	case 16:				// t=11ms
		{
		output5hi;
		OCR0 = poloha5;
		break;
		}
	case 17:
		{
		output5low;
		OCR0 = 253-poloha5;
		break;
		}
	//servo 6
	case 18:				// t=12ms
		{
		output6hi;
		OCR0 = 250;
		break;
		}
	case 19:				// t=13ms
		{
		output6hi;
		OCR0 = poloha6;
		break;
		}
	case 20:
		{
		output6low;
		OCR0 = 253-poloha6;
		break;
		}
	//servo 7
	case 21:				// t=14ms
		{
		output7hi;
		OCR0 = 250;
		break;
		}
	case 22:				// t=15ms
		{
		output7hi;
		OCR0 = poloha7;
		break;
		}
	case 23:
		{
		output7low;
		OCR0 = 253-poloha7;
		break;
		}
	case 24:				// t=16ms				
		{
		OCR0 = 250;
		break;
		}


	//obnovovaci frekvence 50Hz (20ms)
	case 27:				// t=20ms
		{
		timer = 255;
		break;
		}
	}

}//konec preruseni


